@@ -22,10 +22,54 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/arraylist.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <string.h>
 
 
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_LIST (gee_list_get_type ())
+#define GEE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST, GeeList))
+#define GEE_IS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST))
+#define GEE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST, GeeListIface))
+
+typedef struct _GeeList GeeList;
+typedef struct _GeeListIface GeeListIface;
+
+#define GEE_TYPE_ARRAY_LIST (gee_array_list_get_type ())
+#define GEE_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayList))
+#define GEE_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))
+#define GEE_IS_ARRAY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ARRAY_LIST))
+#define GEE_IS_ARRAY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ARRAY_LIST))
+#define GEE_ARRAY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ARRAY_LIST, GeeArrayListClass))
+
+typedef struct _GeeArrayList GeeArrayList;
+typedef struct _GeeArrayListClass GeeArrayListClass;
+typedef struct _GeeArrayListPrivate GeeArrayListPrivate;
+
 #define GEE_ARRAY_LIST_TYPE_ITERATOR (gee_array_list_iterator_get_type ())
 #define GEE_ARRAY_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIterator))
 #define GEE_ARRAY_LIST_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorClass))
@@ -37,30 +81,107 @@ typedef struct _GeeArrayListIterator GeeArrayListIterator;
 typedef struct _GeeArrayListIteratorClass GeeArrayListIteratorClass;
 typedef struct _GeeArrayListIteratorPrivate GeeArrayListIteratorPrivate;
 
-struct _GeeArrayListIterator {
-	GObject parent_instance;
-	GeeArrayListIteratorPrivate * priv;
-	gint _stamp;
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
 };
 
-struct _GeeArrayListIteratorClass {
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GType (*get_element_type) (GeeIterable* self);
+	GeeIterator* (*iterator) (GeeIterable* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gint (*get_size) (GeeCollection* self);
+};
+
+/**
+ * Represents a collection of items in a well-defined order.
+ */
+struct _GeeListIface {
+	GTypeInterface parent_iface;
+	gpointer (*get) (GeeList* self, gint index);
+	void (*set) (GeeList* self, gint index, gconstpointer item);
+	gint (*index_of) (GeeList* self, gconstpointer item);
+	void (*insert) (GeeList* self, gint index, gconstpointer item);
+	void (*remove_at) (GeeList* self, gint index);
+	GeeList* (*slice) (GeeList* self, gint start, gint stop);
+};
+
+/**
+ * Arrays of arbitrary elements which grow automatically as elements are added.
+ */
+struct _GeeArrayList {
+	GObject parent_instance;
+	GeeArrayListPrivate * priv;
+};
+
+struct _GeeArrayListClass {
 	GObjectClass parent_class;
 };
 
-
-
 struct _GeeArrayListPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
 	gpointer* _items;
 	gint _items_length1;
 	gint _items_size;
 	gint _size;
 	GEqualFunc _equal_func;
 	gint _stamp;
+};
+
+struct _GeeArrayListIterator {
+	GObject parent_instance;
+	GeeArrayListIteratorPrivate * priv;
+};
+
+struct _GeeArrayListIteratorClass {
+	GObjectClass parent_class;
+};
+
+struct _GeeArrayListIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
+	GeeArrayList* _list;
+	gint _index;
+	gint _stamp;
 };
 
+
+static gpointer gee_array_list_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_array_list_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_array_list_parent_class = NULL;
+static GeeIterableIface* gee_array_list_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_array_list_gee_collection_parent_iface = NULL;
+static GeeListIface* gee_array_list_gee_list_parent_iface = NULL;
+
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_list_get_type (void);
+GType gee_array_list_get_type (void);
 #define GEE_ARRAY_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ARRAY_LIST, GeeArrayListPrivate))
 enum  {
 	GEE_ARRAY_LIST_DUMMY_PROPERTY,
@@ -70,29 +191,32 @@ enum  {
 	GEE_ARRAY_LIST_G_DUP_FUNC,
 	GEE_ARRAY_LIST_G_DESTROY_FUNC
 };
+void gee_array_list_set_equal_func (GeeArrayList* self, GEqualFunc value);
+GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
+GeeArrayList* gee_array_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
+GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GEqualFunc equal_func);
 static GType gee_array_list_real_get_element_type (GeeIterable* base);
+static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static GType gee_array_list_iterator_get_type (void);
 static GeeIterator* gee_array_list_real_iterator (GeeIterable* base);
+gint gee_list_index_of (GeeList* self, gconstpointer item);
 static gboolean gee_array_list_real_contains (GeeCollection* base, gconstpointer item);
 static gint gee_array_list_real_index_of (GeeList* base, gconstpointer item);
 static gpointer gee_array_list_real_get (GeeList* base, gint index);
 static void gee_array_list_real_set (GeeList* base, gint index, gconstpointer item);
+static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
 static gboolean gee_array_list_real_add (GeeCollection* base, gconstpointer item);
+static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
 static void gee_array_list_real_insert (GeeList* base, gint index, gconstpointer item);
+void gee_list_remove_at (GeeList* self, gint index);
 static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_array_list_real_remove_at (GeeList* base, gint index);
 static void gee_array_list_real_clear (GeeCollection* base);
+gboolean gee_collection_add (GeeCollection* self, gconstpointer item);
+gpointer gee_list_get (GeeList* self, gint index);
 static GeeList* gee_array_list_real_slice (GeeList* base, gint start, gint stop);
-static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta);
-static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count);
 static void gee_array_list_set_capacity (GeeArrayList* self, gint value);
-struct _GeeArrayListIteratorPrivate {
-	GeeArrayList* _list;
-	gint _index;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
 #define GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_ARRAY_LIST_TYPE_ITERATOR, GeeArrayListIteratorPrivate))
 enum  {
 	GEE_ARRAY_LIST_ITERATOR_DUMMY_PROPERTY,
@@ -101,20 +225,17 @@ enum  {
 	GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC,
 	GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC
 };
-static GeeArrayListIterator* gee_array_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
+static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value);
 static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeArrayList* list);
 static gboolean gee_array_list_iterator_real_next (GeeIterator* base);
 static gpointer gee_array_list_iterator_real_get (GeeIterator* base);
-static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value);
-static gpointer gee_array_list_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_array_list_iterator_gee_iterator_parent_iface = NULL;
 static void gee_array_list_iterator_finalize (GObject* obj);
-static GType gee_array_list_iterator_get_type (void);
-static gpointer gee_array_list_parent_class = NULL;
-static GeeIterableIface* gee_array_list_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_array_list_gee_collection_parent_iface = NULL;
-static GeeListIface* gee_array_list_gee_list_parent_iface = NULL;
+static void gee_array_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_array_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_array_list_finalize (GObject* obj);
+static void gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_array_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
 static void _vala_array_move (gpointer array, gsize element_size, gint src, gint dest, gint length);
 
@@ -156,150 +277,164 @@ GeeArrayList* gee_array_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDest
 
 static GType gee_array_list_real_get_element_type (GeeIterable* base) {
 	GeeArrayList * self;
+	GType result;
 	self = (GeeArrayList*) base;
-	return self->priv->g_type;
+	result = self->priv->g_type;
+	return result;
 }
 
 
 static GeeIterator* gee_array_list_real_iterator (GeeIterable* base) {
 	GeeArrayList * self;
+	GeeIterator* result;
 	self = (GeeArrayList*) base;
-	return (GeeIterator*) gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
+	result = (GeeIterator*) gee_array_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
+	return result;
 }
 
 
 static gboolean gee_array_list_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeArrayList * self;
+	gboolean result;
 	self = (GeeArrayList*) base;
-	return (gee_list_index_of ((GeeList*) self, item) != (-1));
+	result = gee_list_index_of ((GeeList*) self, item) != (-1);
+	return result;
 }
 
 
 static gint gee_array_list_real_index_of (GeeList* base, gconstpointer item) {
 	GeeArrayList * self;
+	gint result;
 	self = (GeeArrayList*) base;
 	{
 		gint index;
 		index = 0;
-		for (; index < self->priv->_size; index++) {
-			if (self->priv->_equal_func (self->priv->_items[index], item)) {
-				return index;
+		{
+			gboolean _tmp0_;
+			_tmp0_ = TRUE;
+			while (TRUE) {
+				if (!_tmp0_) {
+					index++;
+				}
+				_tmp0_ = FALSE;
+				if (!(index < self->priv->_size)) {
+					break;
+				}
+				if (self->priv->_equal_func (self->priv->_items[index], item)) {
+					result = index;
+					return result;
+				}
 			}
 		}
 	}
-	return -1;
+	result = -1;
+	return result;
 }
 
 
 static gpointer gee_array_list_real_get (GeeList* base, gint index) {
 	GeeArrayList * self;
-	gboolean _tmp0;
-	gconstpointer _tmp1;
+	gpointer result;
+	gconstpointer _tmp0_;
 	self = (GeeArrayList*) base;
-	_tmp0 = FALSE;
-	if (index >= 0) {
-		_tmp0 = index < self->priv->_size;
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
-	_tmp1 = NULL;
-	return (_tmp1 = self->priv->_items[index], ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1));
+	g_assert (index >= 0);
+	g_assert (index < self->priv->_size);
+	_tmp0_ = NULL;
+	result = (_tmp0_ = self->priv->_items[index], ((_tmp0_ == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g_dup_func ((gpointer) _tmp0_));
+	return result;
 }
 
 
 static void gee_array_list_real_set (GeeList* base, gint index, gconstpointer item) {
 	GeeArrayList * self;
-	gboolean _tmp0;
-	gpointer _tmp2;
-	gconstpointer _tmp1;
+	gpointer _tmp1_;
+	gconstpointer _tmp0_;
 	self = (GeeArrayList*) base;
-	_tmp0 = FALSE;
-	if (index >= 0) {
-		_tmp0 = index < self->priv->_size;
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
-	_tmp2 = NULL;
-	_tmp1 = NULL;
-	self->priv->_items[index] = (_tmp2 = (_tmp1 = item, ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp2);
+	g_assert (index >= 0);
+	g_assert (index < self->priv->_size);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_items[index] = (_tmp1_ = (_tmp0_ = item, ((_tmp0_ == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g_dup_func ((gpointer) _tmp0_)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp1_);
 }
 
 
 static gboolean gee_array_list_real_add (GeeCollection* base, gconstpointer item) {
 	GeeArrayList * self;
-	gint _tmp2;
-	gpointer _tmp1;
-	gconstpointer _tmp0;
+	gboolean result;
+	gpointer _tmp2_;
+	gpointer *_tmp1_;
+	gconstpointer _tmp0_;
 	self = (GeeArrayList*) base;
 	if (self->priv->_size == self->priv->_items_length1) {
 		gee_array_list_grow_if_needed (self, 1);
 	}
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	_tmp2 = self->priv->_size++;
-	self->priv->_items[_tmp2] = (_tmp1 = (_tmp0 = item, ((_tmp0 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0) : self->priv->g_dup_func ((gpointer) _tmp0)), ((self->priv->_items[_tmp2] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[_tmp2] = (self->priv->g_destroy_func (self->priv->_items[_tmp2]), NULL)), _tmp1);
+	_tmp2_ = NULL;
+	_tmp0_ = NULL;
+	_tmp1_ = &self->priv->_items[self->priv->_size++];
+	(*_tmp1_) = (_tmp2_ = (_tmp0_ = item, ((_tmp0_ == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g_dup_func ((gpointer) _tmp0_)), (((*_tmp1_) == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : ((*_tmp1_) = (self->priv->g_destroy_func ((*_tmp1_)), NULL)), _tmp2_);
 	self->priv->_stamp++;
-	return TRUE;
+	result = TRUE;
+	return result;
 }
 
 
 static void gee_array_list_real_insert (GeeList* base, gint index, gconstpointer item) {
 	GeeArrayList * self;
-	gboolean _tmp0;
-	gpointer _tmp2;
-	gconstpointer _tmp1;
+	gpointer _tmp1_;
+	gconstpointer _tmp0_;
 	self = (GeeArrayList*) base;
-	_tmp0 = FALSE;
-	if (index >= 0) {
-		_tmp0 = index <= self->priv->_size;
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
+	g_assert (index >= 0);
+	g_assert (index <= self->priv->_size);
 	if (self->priv->_size == self->priv->_items_length1) {
 		gee_array_list_grow_if_needed (self, 1);
 	}
 	gee_array_list_shift (self, index, 1);
-	_tmp2 = NULL;
-	_tmp1 = NULL;
-	self->priv->_items[index] = (_tmp2 = (_tmp1 = item, ((_tmp1 == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp1) : self->priv->g_dup_func ((gpointer) _tmp1)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp2);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_items[index] = (_tmp1_ = (_tmp0_ = item, ((_tmp0_ == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g_dup_func ((gpointer) _tmp0_)), ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp1_);
 	self->priv->_stamp++;
 }
 
 
 static gboolean gee_array_list_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeArrayList * self;
+	gboolean result;
 	self = (GeeArrayList*) base;
 	{
 		gint index;
 		index = 0;
-		for (; index < self->priv->_size; index++) {
-			if (self->priv->_equal_func (self->priv->_items[index], item)) {
-				gee_list_remove_at ((GeeList*) self, index);
-				return TRUE;
+		{
+			gboolean _tmp0_;
+			_tmp0_ = TRUE;
+			while (TRUE) {
+				if (!_tmp0_) {
+					index++;
+				}
+				_tmp0_ = FALSE;
+				if (!(index < self->priv->_size)) {
+					break;
+				}
+				if (self->priv->_equal_func (self->priv->_items[index], item)) {
+					gee_list_remove_at ((GeeList*) self, index);
+					result = TRUE;
+					return result;
+				}
 			}
 		}
 	}
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
 static void gee_array_list_real_remove_at (GeeList* base, gint index) {
 	GeeArrayList * self;
-	gboolean _tmp0;
-	gpointer _tmp1;
+	gpointer _tmp0_;
 	self = (GeeArrayList*) base;
-	_tmp0 = FALSE;
-	if (index >= 0) {
-		_tmp0 = index < self->priv->_size;
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
-	_tmp1 = NULL;
-	self->priv->_items[index] = (_tmp1 = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp1);
+	g_assert (index >= 0);
+	g_assert (index < self->priv->_size);
+	_tmp0_ = NULL;
+	self->priv->_items[index] = (_tmp0_ = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp0_);
 	gee_array_list_shift (self, index + 1, -1);
 	self->priv->_stamp++;
 }
@@ -311,10 +446,21 @@ static void gee_array_list_real_clear (GeeCollection* base) {
 	{
 		gint index;
 		index = 0;
-		for (; index < self->priv->_size; index++) {
-			gpointer _tmp0;
-			_tmp0 = NULL;
-			self->priv->_items[index] = (_tmp0 = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp0);
+		{
+			gboolean _tmp0_;
+			_tmp0_ = TRUE;
+			while (TRUE) {
+				gpointer _tmp1_;
+				if (!_tmp0_) {
+					index++;
+				}
+				_tmp0_ = FALSE;
+				if (!(index < self->priv->_size)) {
+					break;
+				}
+				_tmp1_ = NULL;
+				self->priv->_items[index] = (_tmp1_ = NULL, ((self->priv->_items[index] == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->priv->_items[index] = (self->priv->g_destroy_func (self->priv->_items[index]), NULL)), _tmp1_);
+			}
 		}
 	}
 	self->priv->_size = 0;
@@ -324,45 +470,46 @@ static void gee_array_list_real_clear (GeeCollection* base) {
 
 static GeeList* gee_array_list_real_slice (GeeList* base, gint start, gint stop) {
 	GeeArrayList * self;
+	GeeList* result;
 	GeeArrayList* slice;
 	self = (GeeArrayList*) base;
 	g_return_val_if_fail (start <= stop, NULL);
 	g_return_val_if_fail (start >= 0, NULL);
-	g_return_val_if_fail (stop <= gee_collection_get_size ((GeeCollection*) self), NULL);
+	g_return_val_if_fail (stop <= self->priv->_size, NULL);
 	slice = gee_array_list_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self->priv->_equal_func);
 	{
 		gint i;
 		i = start;
-		for (; i < stop; i++) {
-			gpointer _tmp0;
-			_tmp0 = NULL;
-			gee_collection_add ((GeeCollection*) slice, _tmp0 = gee_list_get ((GeeList*) self, i));
-			((_tmp0 == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp0 = (self->priv->g_destroy_func (_tmp0), NULL));
+		{
+			gboolean _tmp0_;
+			_tmp0_ = TRUE;
+			while (TRUE) {
+				gpointer _tmp1_;
+				if (!_tmp0_) {
+					i++;
+				}
+				_tmp0_ = FALSE;
+				if (!(i < stop)) {
+					break;
+				}
+				_tmp1_ = NULL;
+				gee_collection_add ((GeeCollection*) slice, _tmp1_ = gee_list_get ((GeeList*) self, i));
+				((_tmp1_ == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (_tmp1_ = (self->priv->g_destroy_func (_tmp1_), NULL));
+			}
 		}
 	}
-	return (GeeList*) slice;
+	result = (GeeList*) slice;
+	return result;
 }
 
 
 static void gee_array_list_shift (GeeArrayList* self, gint start, gint delta) {
-	gboolean _tmp0;
-	gboolean _tmp1;
 	g_return_if_fail (self != NULL);
-	_tmp0 = FALSE;
-	_tmp1 = FALSE;
-	if (start >= 0) {
-		_tmp1 = start <= self->priv->_size;
-	} else {
-		_tmp1 = FALSE;
-	}
-	if (_tmp1) {
-		_tmp0 = start >= (-delta);
-	} else {
-		_tmp0 = FALSE;
-	}
-	g_assert (_tmp0);
+	g_assert (start >= 0);
+	g_assert (start <= self->priv->_size);
+	g_assert (start >= (-delta));
 	_vala_array_move (self->priv->_items, sizeof (gpointer), start, start + delta, self->priv->_size - start);
-	self->priv->_size = self->priv->_size + (delta);
+	self->priv->_size = self->priv->_size + delta;
 }
 
 
@@ -372,34 +519,36 @@ static void gee_array_list_grow_if_needed (GeeArrayList* self, gint new_count) {
 	g_assert (new_count >= 0);
 	minimum_size = self->priv->_size + new_count;
 	if (minimum_size > self->priv->_items_length1) {
-		gint _tmp0;
-		_tmp0 = 0;
+		gint _tmp0_;
+		_tmp0_ = 0;
 		if (new_count > self->priv->_items_length1) {
-			_tmp0 = minimum_size;
+			_tmp0_ = minimum_size;
 		} else {
-			_tmp0 = 2 * self->priv->_items_length1;
+			_tmp0_ = 2 * self->priv->_items_length1;
 		}
 		/* double the capacity unless we add even more items at this time*/
-		gee_array_list_set_capacity (self, _tmp0);
+		gee_array_list_set_capacity (self, _tmp0_);
 	}
 }
 
 
 static void gee_array_list_set_capacity (GeeArrayList* self, gint value) {
-	gint _tmp0;
+	gint _tmp0_;
 	g_return_if_fail (self != NULL);
 	g_assert (value >= self->priv->_size);
-	_tmp0 = value;
+	_tmp0_ = value;
 	self->priv->_items = g_renew (gpointer, self->priv->_items, value);
-	(_tmp0 > self->priv->_items_length1) ? memset (self->priv->_items + self->priv->_items_length1, 0, sizeof (gpointer) * (_tmp0 - self->priv->_items_length1)) : NULL;
-	self->priv->_items_length1 = _tmp0;
+	(_tmp0_ > self->priv->_items_length1) ? memset (self->priv->_items + self->priv->_items_length1, 0, sizeof (gpointer) * (_tmp0_ - self->priv->_items_length1)) : NULL;
+	self->priv->_items_length1 = _tmp0_;
 }
 
 
 static gint gee_array_list_real_get_size (GeeCollection* base) {
+	gint result;
 	GeeArrayList* self;
 	self = (GeeArrayList*) base;
-	return self->priv->_size;
+	result = self->priv->_size;
+	return result;
 }
 
 
@@ -447,45 +596,97 @@ static GeeArrayListIterator* gee_array_list_iterator_new (GType g_type, GBoxedCo
 
 static gboolean gee_array_list_iterator_real_next (GeeIterator* base) {
 	GeeArrayListIterator * self;
+	gboolean result;
 	self = (GeeArrayListIterator*) base;
-	g_assert (self->_stamp == self->priv->_list->priv->_stamp);
+	g_assert (self->priv->_stamp == self->priv->_list->priv->_stamp);
 	if (self->priv->_index < self->priv->_list->priv->_size) {
 		self->priv->_index++;
 	}
-	return (self->priv->_index < self->priv->_list->priv->_size);
+	result = self->priv->_index < self->priv->_list->priv->_size;
+	return result;
 }
 
 
 static gpointer gee_array_list_iterator_real_get (GeeIterator* base) {
 	GeeArrayListIterator * self;
-	gboolean _tmp0;
+	gpointer result;
+	gboolean _tmp0_;
 	self = (GeeArrayListIterator*) base;
-	g_assert (self->_stamp == self->priv->_list->priv->_stamp);
-	_tmp0 = FALSE;
+	g_assert (self->priv->_stamp == self->priv->_list->priv->_stamp);
+	_tmp0_ = FALSE;
 	if (self->priv->_index < 0) {
-		_tmp0 = TRUE;
+		_tmp0_ = TRUE;
 	} else {
-		_tmp0 = self->priv->_index >= self->priv->_list->priv->_size;
+		_tmp0_ = self->priv->_index >= self->priv->_list->priv->_size;
 	}
-	if (_tmp0) {
-		return NULL;
+	if (_tmp0_) {
+		result = NULL;
+		return result;
 	}
-	return gee_list_get ((GeeList*) self->priv->_list, self->priv->_index);
+	result = gee_list_get ((GeeList*) self->priv->_list, self->priv->_index);
+	return result;
 }
 
 
 static void gee_array_list_iterator_set_list (GeeArrayListIterator* self, GeeArrayList* value) {
-	GeeArrayList* _tmp1;
-	GeeArrayList* _tmp0;
+	GeeArrayList* _tmp1_;
+	GeeArrayList* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_list = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)), _tmp1);
-	self->_stamp = self->priv->_list->priv->_stamp;
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_list = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)), _tmp1_);
+	self->priv->_stamp = self->priv->_list->priv->_stamp;
 	g_object_notify ((GObject *) self, "list");
 }
 
 
+static void gee_array_list_iterator_class_init (GeeArrayListIteratorClass * klass) {
+	gee_array_list_iterator_parent_class = g_type_class_peek_parent (klass);
+	g_type_class_add_private (klass, sizeof (GeeArrayListIteratorPrivate));
+	G_OBJECT_CLASS (klass)->get_property = gee_array_list_iterator_get_property;
+	G_OBJECT_CLASS (klass)->set_property = gee_array_list_iterator_set_property;
+	G_OBJECT_CLASS (klass)->finalize = gee_array_list_iterator_finalize;
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_LIST, g_param_spec_object ("list", "list", "list", GEE_TYPE_ARRAY_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
+}
+
+
+static void gee_array_list_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
+	gee_array_list_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
+	iface->next = gee_array_list_iterator_real_next;
+	iface->get = gee_array_list_iterator_real_get;
+}
+
+
+static void gee_array_list_iterator_instance_init (GeeArrayListIterator * self) {
+	self->priv = GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE (self);
+	self->priv->_index = -1;
+	self->priv->_stamp = 0;
+}
+
+
+static void gee_array_list_iterator_finalize (GObject* obj) {
+	GeeArrayListIterator * self;
+	self = GEE_ARRAY_LIST_ITERATOR (obj);
+	(self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL));
+	G_OBJECT_CLASS (gee_array_list_iterator_parent_class)->finalize (obj);
+}
+
+
+static GType gee_array_list_iterator_get_type (void) {
+	static GType gee_array_list_iterator_type_id = 0;
+	if (gee_array_list_iterator_type_id == 0) {
+		static const GTypeInfo g_define_type_info = { sizeof (GeeArrayListIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_array_list_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeArrayListIterator), 0, (GInstanceInitFunc) gee_array_list_iterator_instance_init, NULL };
+		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
+		gee_array_list_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeArrayListIterator", &g_define_type_info, 0);
+		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
+	}
+	return gee_array_list_iterator_type_id;
+}
+
+
 static void gee_array_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	GeeArrayListIterator * self;
 	gpointer boxed;
@@ -521,91 +722,6 @@ static void gee_array_list_iterator_set_property (GObject * object, guint proper
 }
 
 
-static void gee_array_list_iterator_class_init (GeeArrayListIteratorClass * klass) {
-	gee_array_list_iterator_parent_class = g_type_class_peek_parent (klass);
-	g_type_class_add_private (klass, sizeof (GeeArrayListIteratorPrivate));
-	G_OBJECT_CLASS (klass)->get_property = gee_array_list_iterator_get_property;
-	G_OBJECT_CLASS (klass)->set_property = gee_array_list_iterator_set_property;
-	G_OBJECT_CLASS (klass)->finalize = gee_array_list_iterator_finalize;
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ARRAY_LIST_ITERATOR_LIST, g_param_spec_object ("list", "list", "list", GEE_TYPE_ARRAY_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
-}
-
-
-static void gee_array_list_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
-	gee_array_list_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
-	iface->next = gee_array_list_iterator_real_next;
-	iface->get = gee_array_list_iterator_real_get;
-}
-
-
-static void gee_array_list_iterator_instance_init (GeeArrayListIterator * self) {
-	self->priv = GEE_ARRAY_LIST_ITERATOR_GET_PRIVATE (self);
-	self->priv->_index = -1;
-	self->_stamp = 0;
-}
-
-
-static void gee_array_list_iterator_finalize (GObject* obj) {
-	GeeArrayListIterator * self;
-	self = GEE_ARRAY_LIST_ITERATOR (obj);
-	(self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL));
-	G_OBJECT_CLASS (gee_array_list_iterator_parent_class)->finalize (obj);
-}
-
-
-static GType gee_array_list_iterator_get_type (void) {
-	static GType gee_array_list_iterator_type_id = 0;
-	if (gee_array_list_iterator_type_id == 0) {
-		static const GTypeInfo g_define_type_info = { sizeof (GeeArrayListIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_array_list_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeArrayListIterator), 0, (GInstanceInitFunc) gee_array_list_iterator_instance_init, NULL };
-		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_array_list_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
-		gee_array_list_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeArrayListIterator", &g_define_type_info, 0);
-		g_type_add_interface_static (gee_array_list_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
-	}
-	return gee_array_list_iterator_type_id;
-}
-
-
-static void gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeArrayList * self;
-	gpointer boxed;
-	self = GEE_ARRAY_LIST (object);
-	switch (property_id) {
-		case GEE_ARRAY_LIST_SIZE:
-		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
-		break;
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
-static void gee_array_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeArrayList * self;
-	self = GEE_ARRAY_LIST (object);
-	switch (property_id) {
-		case GEE_ARRAY_LIST_EQUAL_FUNC:
-		gee_array_list_set_equal_func (self, g_value_get_pointer (value));
-		break;
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-		case GEE_ARRAY_LIST_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_ARRAY_LIST_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_ARRAY_LIST_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
-	}
-}
-
-
 static void gee_array_list_class_init (GeeArrayListClass * klass) {
 	gee_array_list_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GeeArrayListPrivate));
@@ -680,7 +796,45 @@ GType gee_array_list_get_type (void) {
 }
 
 
-static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+static void gee_array_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeArrayList * self;
+	gpointer boxed;
+	self = GEE_ARRAY_LIST (object);
+	switch (property_id) {
+		case GEE_ARRAY_LIST_SIZE:
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
+static void gee_array_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeArrayList * self;
+	self = GEE_ARRAY_LIST (object);
+	switch (property_id) {
+		case GEE_ARRAY_LIST_EQUAL_FUNC:
+		gee_array_list_set_equal_func (self, g_value_get_pointer (value));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+		case GEE_ARRAY_LIST_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_ARRAY_LIST_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_ARRAY_LIST_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
+	}
+}
+
+
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
 		for (i = 0; i < array_length; i = i + 1) {
@@ -689,6 +843,11 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 			}
 		}
 	}
+}
+
+
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+	_vala_array_destroy (array, array_length, destroy_func);
 	g_free (array);
 }
 
